/* Arduino MinPrintf Library
 * Copyright (C) 2014 by William Greiman
 *
 * This file is part of the Arduino MinPrintf Library
 *
 * This Library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Arduino MinPrintf Library.  If not, see
 * <http://www.gnu.org/licenses/>.
 */
#include <MinPrintf.h>
//------------------------------------------------------------------------------
inline char fmtChr(bool flash, const char* fmt) {
  return flash ? pgm_read_byte(fmt) : *fmt;
}
//------------------------------------------------------------------------------
void MinPrintf::printf(const char* fmt, ...) {
	va_list va;
	va_start(va, fmt);
	vprintf(0, fmt, va);
	va_end(va);
}
//------------------------------------------------------------------------------
void MinPrintf::printf(const __FlashStringHelper* fmtc, ...) {
  va_list va;
  va_start(va, fmtc);
  const char PROGMEM *fmt = (const char PROGMEM *)fmtc;
	vprintf(1, fmt, va);  
	va_end(va);  
 }
 //-----------------------------------------------------------------------------
void MinPrintf::vprintf(bool flash, const char* fmt, va_list va) {
  char ch;
  char len;
  int32_t sval;
  uint32_t uval;
  
 while (ch = fmtChr(flash, fmt++)) {
    if (ch != '%') {
#if NEW_LINE_IS_CRLF
      if (ch == '\n') m_pr->write('\r');
#endif  // NEW_LINE_IS_CRLF
      m_pr->write(ch);
      continue;
    }
    ch = fmtChr(flash, fmt++);
    if (ch == 'l') {
      len = ch;
      ch = fmtChr(flash, fmt++);
    } else {
      len = 0;
    }
    switch (ch) {
    case 'd':
      sval = len ? va_arg(va, long) : va_arg(va, int);
      m_pr->print(sval);
      break;
      
    case 'u':
      uval = len ? va_arg(va, unsigned long) : va_arg(va, unsigned);
      m_pr->print(uval);
      break;
      
#if CHAR_AND_STRING
    case 'c':
      m_pr->write(va_arg(va, int));
      break;

    case 's':
      m_pr->write(va_arg(va, char*));
      break;
#endif CHAR_AND_STRING
    
    default:
      if (ch != '%') m_pr->write('?');
      if (ch == 0) return;
      m_pr->write(ch);
    }
  }
}